#include <avr/io.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <stdlib.h>

void ADC_init(){
	
	PORTA = 0;
	PORTD = 0;
	PORTB = 0;
	
	DDRD |= _BV(PD4) | _BV(PD5);
	DDRB |= _BV(PB3);
	
	ADCSRA = 0;
	ADCSRA |= _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0); //set clock prescaler at 128
	ADCSRA |= _BV(ADEN);	//enable ADC

	ADMUX = 0;
	ADMUX |= 0b11000000;
}

int ADC_get_signal(){
	ADCSRA |= _BV(ADSC);					// start first conversion
	loop_until_bit_is_set(ADCSRA,ADIF);		// wait until conversion is done: until ADIF from ADCSRA is set to 1
	ADCSRA |= _BV(ADIF);
	return (ADC);							// returns the ADC
}


int main(){
	
	int state = -1;
	int num_iters = 0, i;
	int t1 = 0, t2 = 0, t3 = 0;
	
	ADC_init();
	
	while(1){
		// daca se bate din palme(1024 maxim)
		if (ADC_get_signal() > 1022) {
			state = (state + 1) % 5;
			
			switch (state)
			{
			case 0: 
				PORTB |= _BV(PB3); // aprind RED
				_delay_ms(100);
				break;
			case 1: 
				PORTB &= ~(_BV(PB3));
				PORTD |= _BV(PD4);	// aprind GREEN
				_delay_ms(100);
				break;
			case 2:
				PORTD &= ~(_BV(PD4));
				PORTD |= _BV(PD5);	//aprind BLUE
				_delay_ms(100);
				break;
			case 3:
				PORTD &= ~(_BV(PD5));
				
				t1 = rand() % 11;
				t2 = rand() % 11;
				t3 = rand() % 11;
				
				if (t2 < t1) {
					int aux = t1;
					t1 = t2;
					t2 = aux;
				}
				
				if (t3 < t1) {
					int aux = t1;
					t1 = t3;
					t3 = t2;
					t2 = aux;
				}
				else if (t3 < t2) {
					int aux = t2;
					t2 = t3;
					t3 = aux;
				}
				
				for (i = 0; i < 50; i++) {
					PORTB |= _BV(PB3);			//aprind RED
					PORTD |= _BV(PD4);	 		//aprind GREEN
					PORTD |= _BV(PD5);	 		//aprind BLUE
					_delay_ms(t1);
					PORTB &= ~(_BV(PB3)); 		//sting RED
					_delay_ms(t2 - t1);
					PORTD &= ~(_BV(PD4));	 	//sting GREEN
					_delay_ms(t3 - t2);
					PORTD &= ~(_BV(PD5));	 	//sting BLUE
					_delay_ms(10 - t3);
				}
				break;				
				
			case 4:
				PORTD = 0;	// sting toate culorile
				PORTB = 0;
				_delay_ms(100);
				break;
			}
		}
		
		if (state == 3) {
			PORTD &= ~(_BV(PD5));
			
			num_iters = (num_iters + 1) % 200;
			
			if (num_iters == 0) {				
				t1 = rand() % 5;
				t2 = rand() % 5;
				t3 = rand() % 5;
				
				if (t2 < t1) {
					int aux = t1;
					t1 = t2;
					t2 = aux;
				}
				
				if (t3 < t1) {
					int aux = t1;
					t1 = t3;
					t3 = t2;
					t2 = aux;
				}
				else if (t3 < t2) {
					int aux = t2;
					t2 = t3;
					t3 = aux;
				}
			}
		
			PORTB |= _BV(PB3);			//aprind RED
			PORTD |= _BV(PD4);	 		//aprind GREEN
			PORTD |= _BV(PD5);	 		//aprind BLUE
			_delay_ms(t1);
			PORTB &= ~(_BV(PB3)); 		//sting RED
			_delay_ms(t2 - t1);
			PORTD &= ~(_BV(PD4));	 	//sting GREEN
			_delay_ms(t3 - t2);
			PORTD &= ~(_BV(PD5));	 	//sting BLUE
			_delay_ms(5 - t3);
		}
		
	}
	
	return 0;
}
